; Multi-Channel Dimmer Controller

;
; Listens on a 232-bus for commands to execute
;
; Packet format
; BYTE 1
;  1 i4 i3 i2 i1 i0 l1 l0
;
;   i4-i0	DEVICE ID	(Overall device ID)
;   l1-l0	LIGHT ID	(Number of the light within this device)
;
; BYTE 2
;  0 b7-b0
;
;   b7-b0	BRIGHTNESS	(7 bit brightness value)
;




;
; define PIC device type
;
;	device	pic16f84

;
; define config fuses
;
;	config	CP=off,WDT=off,PWRT=off,OSC=rc
	__config _wdt_off & _rc_osc

;
; include PIC register definitions, and some macros
;
;	include "pic16c84.h"
;	include "picmac.h"

	include "../include/p16f84.inc"
	include "../include/picmac_gpasm.h"

#include "../include/uart_3110.h"

;
;                        -----------
;                       |     U     |
;        MAX3110_IRQ/ --|RA2     RA1|-- SPI_DIN
;         MAX3110_CS/ --|RA3     RA0|-- SPI_CLK
;           SPI_DOUT  --|RA4        |
;                       |           | 
;                       |           |
;                       |           |
;     Light 1 triac   --|RB0     RB7|-- RE/DE  
;     Light 2 triac   --|RB1     RB6|-- ZC Detector
;     Light 3 triac   --|RB2     RB5|-- Light 6 triac
;     Light 4 triac   --|RB3     RB4|-- Light 5 triac
;                       |           |
;                        -----------



;
; bit definitions for two LEDs connected to port A bits 0 and 1
; bit masks can be computed from bit numbers with the left shift operator.
;

#define DEVICE_ID 0x09 << 2

#define SPI_CLK	PORTA,0
#define SPI_DIN PORTA,1
#define SPI_DOUT PORTA,4
#define BAUD_RATE_3110 B38400

#define PACKET_SIZE 3

#define MAX3110_IRQ PORTA,2
#define MAX3110_CS	PORTA,3
#define ZC_DETECTOR PORTB,6

#define LIGHT_1_TRIAC PORTB,0
#define LIGHT_2_TRIAC PORTB,1
#define LIGHT_3_TRIAC PORTB,2
#define LIGHT_4_TRIAC PORTB,3
#define LIGHT_5_TRIAC PORTB,4
#define LIGHT_6_TRIAC PORTB,5


; configuration for the MAX3110 UART



;
; define some register file variables
;

	SEGU data

	org	0x0c

temp DS 1
temp_w DS 1
temp_s DS 1

count_low DS 1
count DS 1

oldb DS 1
newb DS 1

brightness DS 1
brightness2 DS 1
brightness3 DS 1
brightness4 DS 1
brightness5 DS 1
brightness6 DS 1

received_first_packet DS 1	; remains 0 until first packet is received

; latches for the brightness vars
ldim1 DS 1
ldim2 DS 1
ldim3 DS 1
ldim4 DS 1
ldim5 DS 1
ldim6 DS 1

config1		DS 1
config2		DS 1

direction DS 1

packet_count DS 1
packet_buf DS 3

temp1 DS 1
temp2 DS 1

zero_crossing_occurred DS 1
period	DS 1

have_uart DS 1	; this is set to 1 if the UART initialization succeeds...

	SEG code

;
; code start
;
	org	0
	goto start

;;---------
;; INTERRUPT ROUTINE

	org 4
	save_w_stat

	btfss	ZC_DETECTOR
	goto	irq_end

	movf	TMR0,W
	addlw	0xfb
	movwf	period

	bsf		zero_crossing_occurred,0

irq_end
	bcf		INTCON,RBIF

	restore_w_stat
 	retfie


;; -----------
;; START OF CODE

start
	movlw	0xff
	call	delay

	clrf	count

	clrf	PORTA
	clrf	PORTB	

	bsf	STATUS,RP0			;register page 1
	movlw	B'00010100'			;a4,a2 input, all others output
	movwf	TRISA				;other PORTA pins as inputs
	movlw	B'01000000'			;b6 input, all others output
	movwf	TRISB

	; set options
	; 7 : 1 PORTB pull-ups are disabled
	; 6 : 1 Interrupt on rising esdge of RB0
	; 5 : 0 TMR0 on internal clock
	; 4 : 1 increment on high to low transition at RA4 for external clock
	; 3 : 0 prescaler assigned to TMR0
	; 2-0 : 110  1:128 timer prescale
	movlw	B'11010110'
	movwf	OPTION_REG

	bcf	STATUS,RP0			;register page 0


; turn off timer overflow interrupt	
	bcf INTCON,T0IE

; turn on RB0 edge interrupt
;	bsf INTCON,INTE

	bsf	INTCON, RBIE

; globally enable interrupts
	bsf	INTCON,GIE

#define MY_UART_CONFIG WRITE_CONFIGURATION | FIFO_INT_ENABLED | B38400 | FIFO_DISABLED | PARITY_ENABLED

	bsf		have_uart,0		; assume that we have it

	; we will try to initialize the UART 10 times
	movlw	10
	movwf	temp1

try_to_init_uart
	movlw	HIGH(MY_UART_CONFIG)
	movwf	uart_3110_temp
	movlw	LOW(MY_UART_CONFIG)
	movwf	uart_3110_temp2
	call	UART_232_INIT_PARM

	addlw	0				; test to see if the result was 0
	btfsc	STATUS,Z		; need to see if this is right polarity
	goto	uart_is_good
	
	decfsz	temp1,F
	goto	try_to_init_uart
	
	bcf		have_uart,0		; uart init failed 10 times, so we will assume it
							; does not exist
	
uart_is_good	

	btfss	have_uart,0
	goto	skip_transmit
	
	movlw	'J'
	call	UART_232_TX
	movlw	'J'
	call	UART_232_TX
	movlw	'P'
	call	UART_232_TX

skip_transmit
	clrf	received_first_packet
	clrf	config1
	clrf	config2
	
	movlw	1
	movwf	ldim1
	movlw	11
	movwf	ldim2
	movlw	23
	movwf	ldim3
	movlw	35
	movwf	ldim4
	movlw	47
	movwf	ldim5
	movlw	60
	movwf	ldim6
main_loop_packet_clear
	clrf	packet_count

main_loop
	;; checking all the lights uses 24 cycles
	; check the first light
	movf	brightness,W
	subwf	TMR0,W
	btfsc	STATUS,C
	bsf		LIGHT_1_TRIAC
	
	; check to see if light 2 should be pulsed
	movf	brightness2,W
	subwf	TMR0,W
	btfsc	STATUS,C
	bsf		LIGHT_2_TRIAC

	; check to see if light 3 should be pulsed
	movf	brightness3,W
	subwf	TMR0,W
	btfsc	STATUS,C
	bsf		LIGHT_3_TRIAC

	; check to see if light 4 should be pulsed
	movf	brightness4,W
	subwf	TMR0,W
	btfsc	STATUS,C
	bsf		LIGHT_4_TRIAC

	; check to see if light 5 should be pulsed
	movf	brightness5,W
	subwf	TMR0,W
	btfsc	STATUS,C
	bsf		LIGHT_5_TRIAC

	; check to see if light 6 should be pulsed
	movf	brightness6,W
	subwf	TMR0,W
	btfsc	STATUS,C
	bsf		LIGHT_6_TRIAC

main_loop_partial ; this stub uses 9 in the normal case, 5 if receiving byte

	btfsc	zero_crossing_occurred,0
	call	handle_zero_crossing
	
	btfss	MAX3110_IRQ
	goto	read_data
	
	movf	period,W
	subwf	TMR0,W
	btfss	STATUS,C
	goto	main_loop
	
end_of_cycle
	movf	PORTB,W
	andlw	B'11000000'
	movwf	PORTB
	goto	main_loop_partial
	
	
	
read_data

	; here means the UART is signalling us with an interrupt...
;read byte  7+SPI_READ16(74)+3+UART_232_PARITY_CHECK(20)+6,10=110,114

; the following is an inlined version of 232_RX

    bcf     SPI_DIN 
    bcf     MAX3110_CS
    call    SPI_READ16
    bsf     MAX3110_CS
    btfss   spi_high,7
	goto	main_loop	; if no byte was received
    btfsc   spi_high,2
	goto	main_loop_packet_clear	; if framing error occurred


	call	UART_232_PARITY_CHECK
	addlw	0
	btfss	STATUS,Z
	goto	main_loop_packet_clear
	

	movf	packet_count,F
	btfss	STATUS,Z
	goto	add_to_packet
	
	btfss	spi_low,7
	goto	main_loop
	

; here we are receiving the first byte of a new packet	
address_byte ; (6)
	movlw	1
	movwf	packet_count
	movf	spi_low,W
	movwf	packet_buf
	
;	movlw	'A'
;	call	UART_232_TX
	goto	main_loop

; here we are adding to an already completed packet	
add_to_packet  ; (13)
	movlw	packet_buf
	movwf	FSR
	movf	packet_count,W
	addwf	FSR,F
	movf	spi_low,W
	movwf	IND0
	

;	movlw	'I'
;	call	UART_232_TX

	incf	packet_count,F
	movf	packet_count,W
	sublw	PACKET_SIZE

	btfss	STATUS,Z
	goto	main_loop
	
packet_is_complete ; 7,20
;	movlw	'C'
;	call	UART_232_TX
	clrf	packet_count
	bsf		received_first_packet,0

	; here we see if the packet belongs to us, if it does
	; then we should act on it
	movf	packet_buf,W
	andlw	B'01111100'
	sublw	DEVICE_ID
	btfss	STATUS,Z
	goto	main_loop	
	
;	movlw	'M'
;	call	UART_232_TX

	; the packet is for us...
	movlw	ldim1
	movwf	FSR
	
	movlw	B'00000011'
	andwf	packet_buf,F
	bcf		STATUS,C
	rlf		packet_buf,W
	addwf	FSR,F

;	movf	FSR,W
;	call	UART_232_TX
	
	movf	packet_buf+1,W
	movwf	IND0

;	call	UART_232_TX

	incf	FSR,F

	movf	packet_buf+2,W
	movwf	IND0
	
;	call	UART_232_TX
	
	goto	main_loop	

;--------------
; handle_zero_crossing is called from the main program anytime
; a zero crossing pulse is detected in the interrupt routine

handle_zero_crossing
	clrf	zero_crossing_occurred
	clrf	TMR0

	movf	PORTB,W
	andlw	B'11000000'
	movwf	PORTB

	movf	ldim1,W
	andlw	B'01111111'
	movwf	brightness
	movf	ldim2,W
	andlw	B'01111111'
	movwf	brightness2
	movf	ldim3,W
	andlw	B'01111111'
	movwf	brightness3
	movf	ldim4,W
	andlw	B'01111111'
	movwf	brightness4
	movf	ldim5,W
	andlw	B'01111111'
	movwf	brightness5
	movf	ldim6,W
	andlw	B'01111111'
	movwf	brightness6

	btfsc	received_first_packet,0
	goto	no_auto_inc
	
	movf	count,W
	addlw	16
	movwf	count
	btfss	STATUS,Z
	goto	no_auto_inc
auto_inc
	decf	ldim1,F
	decf	ldim2,F
	decf	ldim3,F
	
	btfss	have_uart,0
	goto 	inc_second_half
	decf	ldim4,F
	decf	ldim5,F
	decf	ldim6,F
	goto	no_auto_inc
	
inc_second_half
	incf	ldim4,F
	incf	ldim5,F
	incf	ldim6,F
no_auto_inc	
	return
	
	org 0x300


#include "spi.asm"

#include "../include/uart_3110.asm"

#include "../include/delay.asm"
	end
